package com.redhat.thermostat.tools.eclipse.plugin.editor;

import java.io.IOException;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.redhat.thermostat.tools.eclipse.plugin.BundleInformation;
import com.redhat.thermostat.tools.eclipse.plugin.BundleInformationExtractor;
import com.redhat.thermostat.tools.eclipse.plugin.Messages;

/**
 * Prompt the user to enter or select a bundle name/version.
 *
 * Used by the command edit page.
 */
public class NewBundleDialog extends TitleAreaDialog {

    private BundleInformation bundleInfo = new BundleInformation();

    private ThermostatBundleInformation thermostatInfo = new ThermostatBundleInformation();

    private Button thermostatBundlesSelectionButton;
    private Combo thermostatVersion;
    private Combo thermostatBundle;

    private Button manualSelectionButton;
    private Text customNameText;
    private Text customVersionText;

    private Button fromJarButton;
    private Text fileLocationText;
    private Button btnFromFile;

    public NewBundleDialog(Shell parentShell) {
        super(parentShell);
    }

    @Override
    public void create() {
        super.create();

        setTitle(Messages.NewBundleDialog_title);
        setMessage(Messages.NewBundleDialog_message);
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite area = (Composite) super.createDialogArea(parent);

        Composite container = new Composite(area, SWT.NONE);
        container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,true));

        GridLayout containerLayout = new GridLayout(3, false);
        containerLayout.marginTop = 5;
        containerLayout.marginRight = 5;
        containerLayout.marginLeft = 5;
        container.setLayout(containerLayout);

        thermostatBundlesSelectionButton = new Button(container, SWT.RADIO);
        thermostatBundlesSelectionButton.setSelection(true);
        thermostatBundlesSelectionButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                thermostatVersion.setEnabled(true);
                thermostatBundle.setEnabled(true);
            }
        });
        thermostatBundlesSelectionButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        thermostatBundlesSelectionButton.setText("Thermostat Bundle");

        Label thermostatVersionLabel = new Label(container, SWT.None);
        thermostatVersionLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        thermostatVersionLabel.setText("Version");

        thermostatVersion = new Combo(container, SWT.None);
        GridData thermostatVersionLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1);
        thermostatVersion.setLayoutData(thermostatVersionLayoutData);
        thermostatVersion.setItems(toArray(thermostatInfo.getVersions()));
        thermostatVersion.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                String selectedVersion = thermostatVersion.getItem(thermostatVersion.getSelectionIndex());
                thermostatBundle.setItems(toArray(thermostatInfo.getBundleNames(selectedVersion)));
            }
        });

        Label thermostatBundleLabel = new Label(container, SWT.None);
        thermostatBundleLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        thermostatBundleLabel.setText("Bundle");

        thermostatBundle = new Combo(container, SWT.NONE);
        GridData thermostatBundleLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1);
        thermostatBundle.setLayoutData(thermostatBundleLayoutData);

        manualSelectionButton = new Button(container, SWT.RADIO);
        manualSelectionButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                customNameText.setEnabled(true);
                customVersionText.setEnabled(true);
            }
        });
        manualSelectionButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        manualSelectionButton.setText(Messages.NewBundleDialog_manualSectionTitle);

        Label nameLabel = new Label(container, SWT.None);
        nameLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        nameLabel.setText(Messages.NewBundleDialog_symbolicNameLabel);
        customNameText = new Text(container, SWT.None);
        GridData nameTextLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
        nameTextLayoutData.horizontalSpan = 2;
        customNameText.setLayoutData(nameTextLayoutData);

        Label versionLabel = new Label(container, SWT.None);
        versionLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        versionLabel.setText(Messages.NewBundleDialog_versionLabel);
        customVersionText = new Text(container, SWT.None);
        GridData versionTextLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
        versionTextLayoutData.horizontalSpan = 2;
        customVersionText.setLayoutData(versionTextLayoutData);

        fromJarButton = new Button(container, SWT.RADIO);
        fromJarButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                fileLocationText.setEnabled(true);
                btnFromFile.setEnabled(true);
            }
        });
        fromJarButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        fromJarButton.setText(Messages.NewBundleDialog_fromJarSectionTitle);

        Label locationLabel = new Label(container, SWT.NONE);
        locationLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        locationLabel.setText(Messages.NewBundleDialog_pathLabel);

        fileLocationText = new Text(container, SWT.BORDER);
        fileLocationText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        btnFromFile = new Button(container, SWT.NONE);
        btnFromFile.setText(Messages.NewBundleDialog_findOnDisk);
        btnFromFile.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                FileDialog dialog = new FileDialog(getShell(), SWT.OPEN);
                dialog.setFilterExtensions(new String[] { "*.jar" }); //$NON-NLS-1$
                String result = dialog.open();
                if (result != null) {
                    fileLocationText.setText(result);
                }
            }
        });

        // invoke manually on startup to enable/disable other widgets
        thermostatBundlesSelectionButton.setSelection(true);

        return area;
    }

    private void disableInputWidgets() {
        thermostatVersion.setEnabled(false);
        thermostatBundle.setEnabled(false);

        customNameText.setEnabled(false);
        customVersionText.setEnabled(false);

        fileLocationText.setEnabled(false);
        btnFromFile.setEnabled(false);
    }

    private String[] toArray(List<String> inputList) {
        return inputList.toArray(new String[inputList.size()]);
    }


    @Override
    protected void okPressed() {
        saveInput();
        super.okPressed();
    }

    private void saveInput() {
        if (thermostatBundlesSelectionButton.getSelection()) {
            bundleInfo = new BundleInformation(thermostatBundle.getText(), thermostatVersion.getText());
        } else if (manualSelectionButton.getSelection()) {
            bundleInfo = new BundleInformation(customNameText.getText(), customVersionText.getText());
        } else if (fromJarButton.getSelection()) {
            try {
                BundleInformationExtractor extractor = new BundleInformationExtractor();
                BundleInformation extractedInfo = extractor.extract(fileLocationText.getText());
                bundleInfo.setName(extractedInfo.getName());
                bundleInfo.setVersion(extractedInfo.getVersion());
            } catch (final IOException e) {
                getShell().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        MessageDialog.openError(getShell(), Messages.NewBundleDialog_errorExtractingInformationFromJarFile, e.getMessage());
                    }
                });
            }
        } else {
            throw new AssertionError("Unexpected case");
        }
    }

    public BundleInformation getBundleInformation() {
        return bundleInfo;
    }
}
